package org.dataone.client.audit;

import java.io.InputStream;

import org.dataone.client.CNode;
import org.dataone.service.cn.v1.CNCore;
import org.dataone.service.exceptions.InsufficientResources;
import org.dataone.service.exceptions.InvalidRequest;
import org.dataone.service.exceptions.InvalidToken;
import org.dataone.service.exceptions.NotAuthorized;
import org.dataone.service.exceptions.NotFound;
import org.dataone.service.exceptions.NotImplemented;
import org.dataone.service.exceptions.ServiceFailure;
import org.dataone.service.types.v1.Checksum;
import org.dataone.service.types.v1.ChecksumAlgorithmList;
import org.dataone.service.types.v1.Identifier;
import org.dataone.service.types.v1.NodeList;
import org.dataone.service.types.v1.NodeReference;
import org.dataone.service.types.v1.ObjectFormat;
import org.dataone.service.types.v1.ObjectFormatIdentifier;
import org.dataone.service.types.v1.ObjectFormatList;
import org.dataone.service.types.v1.ObjectList;
import org.dataone.service.types.v1.ObjectLocationList;
import org.dataone.service.types.v1.Subject;
import org.dataone.service.types.v1.SubjectInfo;
import org.dataone.service.util.D1Url;

/**
 * A D1Node subclass that only provides selected methods from the DataONE CN api
 * that will not change the state of the production machines (other than possibly
 * adding log events)
 * 
 * Usage:  AuditCNode cna = new AuditCNode(baseUrl);
 * 	or:    D1Node cna = new AuditCNode(baseUrl);
 * 
 * @author rnahf
 *
 */
public class AuditCNode extends D1NodeAuditingWrapper {

	private CNode cn;
	
	public AuditCNode(String cnBaseUrl) {
		super(cnBaseUrl);
		this.cn = new CNode(cnBaseUrl);
	}

	public String getNodeBaseServiceUrl() {
		D1Url url = new D1Url(super.getNodeBaseServiceUrl());
		url.addNextPathElement(CNCore.SERVICE_VERSION);
		return url.getUrl();
	}
	
	
	
	public InputStream get(Identifier pid) 
	throws InvalidToken, ServiceFailure, NotAuthorized, NotFound, NotImplemented 
	{
		InputStream is = null;
		try {
			 is = super.get(pid);
		} catch (InsufficientResources e) {
			; // the cn method doesn't throw this exception	
		}
		return is;
	}

	
	// these methods pass-through to the wrapped CNode instance passed in the constructor
	// == these are "wrapper" methods
	
	public ObjectLocationList resolve(Identifier pid) 
	throws InvalidToken, ServiceFailure, NotAuthorized, NotFound, NotImplemented
	{
		return cn.resolve(pid);
	}
	
	
	public ObjectFormatList listFormats() 
	throws ServiceFailure, NotImplemented 
	{
		return cn.listFormats();
	}
	
	public ObjectFormat getFormat(ObjectFormatIdentifier formatId) 
	throws ServiceFailure, NotFound, NotImplemented 
	{
		return cn.getFormat(formatId);
	}
	
	
	public NodeList listNodes() 
	throws NotImplemented, ServiceFailure 
	{
		return cn.listNodes();
	}
	
	
	public Checksum getChecksum(Identifier pid) 
	throws InvalidToken, ServiceFailure, NotAuthorized, NotFound, NotImplemented 
	{
		return cn.getChecksum(pid);
	}
	
	public SubjectInfo getSubjectInfo(Subject subject) 
	throws ServiceFailure, NotAuthorized, NotImplemented, NotFound, InvalidToken
	{
		return cn.getSubjectInfo(subject);
	}
	
	
	public ChecksumAlgorithmList listChecksumAlgorithms() 
	throws ServiceFailure, NotImplemented
	{
		return cn.listChecksumAlgorithms();
	}
	
	
	public ObjectList search(String queryType, String query) 
	throws InvalidToken, ServiceFailure, NotAuthorized, InvalidRequest, NotImplemented 
	{
		return cn.search(queryType, query);
	}
	
	public ObjectList search(String queryType, D1Url query) 
	throws InvalidToken, ServiceFailure, NotAuthorized, InvalidRequest, NotImplemented 
	{
		return cn.search(queryType, query);
	}
	
	
	public SubjectInfo listSubjects(String query, String status, Integer start, Integer count) 
	throws InvalidRequest, ServiceFailure, InvalidToken, NotAuthorized, NotImplemented
	{
		return cn.listSubjects(query, status, start, count);
	}
	
	public String lookupNodeBaseUrl(String nodeId) 
	throws ServiceFailure, NotImplemented 
	{
		return cn.lookupNodeBaseUrl(nodeId);
	}
	
	public String lookupNodeBaseUrl(NodeReference nodeId) 
	throws ServiceFailure, NotImplemented 
	{
		return cn.lookupNodeBaseUrl(nodeId);
	}

}
